import random
import string

import requests
from flask import Request
from flask import Response


class IntrospectionMiddleware:
    def __init__(self, app, scope=None, admin_url=None, login_url=None, callback_url=None):
        self.app = app
        self.login_url = login_url
        self.scope = scope
        self.admin_url = admin_url
        self.callback_url = callback_url

    def get_access_token(self, request):
        header = request.headers.get("Authorization")
        if not header:
            access_token = request.args.get("access_token", "")
            if access_token:
                return access_token

            access_token = request.form.get("access_token", "")
            if access_token:
                return access_token

            return None

        parts = header.split()
        return parts[1]

    def generate_state(self):
        alphabet = string.ascii_lowercase + string.digits + string.ascii_uppercase
        return ''.join(random.choice(alphabet) for i in range(32))

    def __call__(self, environ, start_response):
        request = Request(environ)
        token = self.get_access_token(request)

        if request.base_url in [self.login_url, self.callback_url]:
            return self.app(environ, start_response)

        if not token:
            response = Response()
            response.status_code = 302
            response.headers = [("Location", self.login_url)]
            return response(environ, start_response)

        resp = requests.post(
            f"{self.admin_url}/oauth2/introspect",
            data={
                "scope": self.scope,
                "token": token,
            },
        )
        if resp.status_code != 200:
            response = Response()
            response.status_code = 302
            response.headers = [("Location", self.login_url)]
            return response(environ, start_response)

        if resp.json().get('active', False):
            # FIXME: Handle it
            pass

        return self.app(environ, start_response)


class AccessControlMiddlware:
    def __init__(self, app):
        self.app = app

    def __call__(self, environ, start_response):
        return self.app(environ, start_response)



import json

import requests
from flask import abort
from flask import request


class Authentication:
    def __init__(self, api_url):
        self.api_url = api_url

    def set_email_to_session(self, session) -> None:
        if session.get('email', None):
            return

        resp = requests.get(
            f"{self.api_url}/sessions/whoami",
            cookies=request.cookies,
            headers={"Authorization": request.headers.get("Authorization", "")},
        )
        if resp.status_code == 200:
            data = resp.json()
            identity = data.get("identity", {})
            session["email"] = identity.get("traits", {}).get("email")
            session["kratos_id"] = identity.get("id")
            session["traits"] = json.dumps(identity.get("traits", {}))

    def set_user_to_session(self, session) -> None:
        self.set_email_to_session(session)

    def get_current_user(self, session):
        user_id = session.get("user_id", None)
        if not user_id:
            abort(403)

        return user_id



import requests


class AccessControl:
    def __init__(self, keto_write_url, keto_read_url):
        self.keto_write_url = keto_write_url
        self.keto_read_url = keto_read_url

    def is_allowed(self, namespace, obj, relation, subject_id):
        return self.check_permissions(namespace, obj, relation, subject_id)

    def check_permissions(self, namespace, obj, relation, subject_id) -> bool:
        r = requests.post(
            f"{self.keto_read_url}/check",
            json={
                "object": obj,
                "relation": relation,
                "subject_id": subject_id,
                "namespace": namespace,
            },
        )
        data = r.json()
        return data.get('allowed')

    def add_permissions(self, namespace, obj, relation, subject_id):
        r = requests.put(
            f"{self.keto_write_url}/relation-tuples",
            data={"object": obj, "relation": relation, "subject_id": subject_id, "namespace": namespace},
        )
        r.json()



import requests
from flask import Request
from flask import Response


class AuthenticationMiddleware:
    def __init__(self, app, api_url, ui_url):
        self.app = app
        self.api_url = api_url
        self.ui_url = ui_url

    def __call__(self, environ, start_response):
        request = Request(environ)
        resp = requests.get(
            f"{self.api_url}/sessions/whoami",
            cookies=request.cookies,
            headers={"Authorization": request.headers.get("Authorization", "")},
        )
        if resp.status_code != 200:
            response = Response()
            response.status_code = 302
            response.headers = [("Location", self.ui_url)]
            return response(environ, start_response)

        return self.app(environ, start_response)



import ory_hydra_client
from ory_hydra_client.api import admin_api
from ory_hydra_client.model.accept_consent_request import AcceptConsentRequest
from ory_hydra_client.model.accept_login_request import AcceptLoginRequest
from ory_hydra_client.model.consent_request_session import ConsentRequestSession


class HydraClient:
    def __init__(self, admin_url, public_url):
        self.admin_url = admin_url
        self.public_url = public_url

    def get_login_request(self, login_challenge):
        conf = ory_hydra_client.Configuration(host=self.admin_url)

        with ory_hydra_client.ApiClient(conf) as api_client:
            api_instance = admin_api.AdminApi(api_client)
            return api_instance.get_login_request(login_challenge)

    def accept_login_request(self, login_challenge, traits):
        conf = ory_hydra_client.Configuration(host=self.admin_url)

        with ory_hydra_client.ApiClient(conf) as api_client:
            api_instance = admin_api.AdminApi(api_client)
            return api_instance.accept_login_request(
                login_challenge,
                accept_login_request=AcceptLoginRequest(
                    subject=traits,
                    remember=True,
                    remember_for=3600,
                ),
            )

    def get_consent_request(self, consent_challenge):
        conf = ory_hydra_client.Configuration(host=self.admin_url)

        with ory_hydra_client.ApiClient(conf) as api_client:
            api_instance = admin_api.AdminApi(api_client)
            return api_instance.get_consent_request(
                consent_challenge,
            )

    def accept_consent_request(self, consent_challenge, scope, identity_id):
        conf = ory_hydra_client.Configuration(host=self.admin_url)

        with ory_hydra_client.ApiClient(conf) as api_client:
            api_instance = admin_api.AdminApi(api_client)
            return api_instance.accept_consent_request(
                consent_challenge,
                accept_consent_request=AcceptConsentRequest(
                    grant_scope=scope,
                    remember=True,
                    remember_for=3600,
                    session=ConsentRequestSession(id_token={"id": identity_id}),
                ),
            )

    def reject_consent_request(self, consent_challenge):
        conf = ory_hydra_client.Configuration(host=self.admin_url)

        with ory_hydra_client.ApiClient(conf) as api_client:
            api_instance = admin_api.AdminApi(api_client)
            return api_instance.reject_consent_request(consent_challenge)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['flask_ory_auth',
 'flask_ory_auth.hydra',
 'flask_ory_auth.keto',
 'flask_ory_auth.kratos']

package_data = \
{'': ['*']}

install_requires = \
['Flask>=2.1.2,<3.0.0',
 'requests-oauthlib>=1.3.1,<2.0.0',
 'requests>=2.28.0,<3.0.0']

setup_kwargs = {
    'name': 'flask-ory-auth',
    'version': '0.1.7',
    'description': 'Useful middlewares and clients to use Ory products in flask App',
    'long_description': None,
    'author': 'Andrew Minkin',
    'author_email': 'minkin.andrew@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7',
}


setup(**setup_kwargs)
